@@ -1,13 +1,23 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 '''
 
 '''
-from __future__ import print_function
-
+# Standard library imports
+import os
+import re
 import shutil
+import subprocess
+import sys
+import time
 from glob import glob
 from os.path import dirname, exists, join, realpath, relpath
-import os, re, subprocess, sys, time
 
+# Bokeh imports
 import versioneer
 
 # provide fallbacks for highlights in case colorama is not installed
@@ -28,10 +38,6 @@ except ImportError:
     def green(text) :  return text
     def yellow(text) : return text
 
-# some functions prompt for user input, handle input vs raw_input (py2 vs py3)
-if sys.version_info[0] < 3:
-    input = raw_input # NOQA
-
 # -----------------------------------------------------------------------------
 # Module global variables
 # -----------------------------------------------------------------------------
@@ -39,7 +45,6 @@ if sys.version_info[0] < 3:
 ROOT = dirname(realpath(__file__))
 BOKEHJSROOT = join(ROOT, 'bokehjs')
 BOKEHJSBUILD = join(BOKEHJSROOT, 'build')
-CSS = join(BOKEHJSBUILD, 'css')
 JS = join(BOKEHJSBUILD, 'js')
 SERVER = join(ROOT, 'bokeh/server')
 TSLIB = join(BOKEHJSROOT , 'node_modules/typescript/lib')
@@ -155,24 +160,22 @@ def conda_rendering():
     return os.getenv("CONDA_BUILD_STATE" ,"junk") == "RENDER"
 
 
-def fixup_building_sdist():
-    ''' Check for 'sdist' and ensure we always build BokehJS when packaging
+def check_building_sdist():
+    ''' Check for 'sdist' and ensure we always build or install BokehJS when
+    packaging
 
     Source distributions do not ship with BokehJS source code, but must ship
-    with a pre-built BokehJS library. This function modifies ``sys.argv`` as
-    necessary so that ``--build-js`` IS present, and ``--install-js` is NOT.
+    with a pre-built BokehJS library. This function checks ``sys.argv`` to
+    ensure that ``--build-js`` or ``--install-js` is present.
 
     Returns:
         None
 
     '''
     if "sdist" in sys.argv:
-        if "--install-js" in sys.argv:
-            print("Removing '--install-js' incompatible with 'sdist'")
-            sys.argv.remove('--install-js')
-        if "--build-js" not in sys.argv:
-            print("Adding '--build-js' required for 'sdist'")
-            sys.argv.append('--build-js')
+        if "--install-js" not in sys.argv and "--build-js" not in sys.argv:
+            print("Error: Option '--build-js' or '--install-js' must be present with 'sdist', exiting.")
+            sys.exit(1)
 
 def fixup_for_packaged():
     ''' If we are installing FROM an sdist, then a pre-built BokehJS is
@@ -252,10 +255,10 @@ def jsbuild_prompt():
     '''
     print(BOKEHJS_BUILD_PROMPT)
     mapping = {"1": True, "2": False}
-    value = input("Choice? ")
+    value = input("Choice? ")  # lgtm [py/use-of-input]
     while value not in mapping:
         print("Input '%s' not understood. Valid choices: 1, 2\n" % value)
-        value = input("Choice? ")
+        value = input("Choice? ")  # lgtm [py/use-of-input]
     return mapping[value]
 
 # -----------------------------------------------------------------------------
@@ -263,8 +266,7 @@ def jsbuild_prompt():
 # -----------------------------------------------------------------------------
 
 def build_js():
-    ''' Build BokehJS files (CSS, JS, etc) under the ``bokehjs`` source
-    subdirectory.
+    ''' Build BokehJS files under the ``bokehjs`` source subdirectory.
 
     Also prints a table of statistics about the generated assets (file sizes,
     etc.) or any error messages if the build fails.
@@ -308,7 +310,6 @@ def build_js():
         if not m: continue # skip generate.py output lines
         stamp, txt = m.groups()
         indented_msg += "   " + dim(green(stamp)) + " " + dim(txt) + "\n"
-    msg = "\n".join("    " + x for x in msg.split("\n"))
     print(BUILD_SUCCESS_MSG % indented_msg)
     print("Build time: %s" % bright(yellow("%0.1f seconds" % (t1-t0))))
     print()
@@ -340,14 +341,11 @@ def install_js():
 
     '''
     target_jsdir = join(SERVER, 'static', 'js')
-    target_cssdir = join(SERVER, 'static', 'css')
     target_tslibdir = join(SERVER, 'static', 'lib')
 
     STATIC_ASSETS = [
         join(JS,  'bokeh.js'),
         join(JS,  'bokeh.min.js'),
-        join(CSS, 'bokeh.css'),
-        join(CSS, 'bokeh.min.css'),
     ]
     if not all(exists(a) for a in STATIC_ASSETS):
         print(BOKEHJS_INSTALL_FAIL)
@@ -357,10 +355,6 @@ def install_js():
         shutil.rmtree(target_jsdir)
     shutil.copytree(JS, target_jsdir)
 
-    if exists(target_cssdir):
-        shutil.rmtree(target_cssdir)
-    shutil.copytree(CSS, target_cssdir)
-
     if exists(target_tslibdir):
         shutil.rmtree(target_tslibdir)
     if exists(TSLIB):
@@ -421,7 +415,7 @@ ERROR: subprocess.Popen(%r) failed to execute:
 
     %s
 
-Have you run `npm install --no-save` from the bokehjs subdirectory?
+Have you run `npm ci` from the bokehjs subdirectory?
 For more information, see the Dev Guide:
 
     https://docs.bokeh.org/en/latest/docs/dev_guide.html
