@@ -1,7 +1,6 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2018, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
@@ -13,28 +12,20 @@ and also deliver this capability with high-performance interactivity over large
 or streaming datasets. Bokeh can help anyone who would like to quickly and
 easily create interactive plots, dashboards, and data applications.
 
-For full documentation, please visit: https://bokeh.pydata.org
+For full documentation, please visit: https://docs.bokeh.org
 
 '''
 
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
-# Bokeh imports
-
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -72,20 +63,20 @@ def license():
 # Code
 #-----------------------------------------------------------------------------
 
-# configure Bokeh version
+# expose Bokeh version
 from .util.version import __version__; __version__
 
 # expose sample data module
 from . import sampledata; sampledata
 
 # configure Bokeh logger
-from .util import logconfig
+from .util import logconfig # isort:skip
 del logconfig
 
 # Configure warnings to always show nice mssages, despite Python's active
 # efforts to hide them from users.
-import warnings
-from .util.warnings import BokehDeprecationWarning, BokehUserWarning
+import warnings # isort:skip
+from .util.warnings import BokehDeprecationWarning, BokehUserWarning # isort:skip
 warnings.simplefilter('always', BokehDeprecationWarning)
 warnings.simplefilter('always', BokehUserWarning)
 
