@@ -1,7 +1,6 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2018, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
@@ -20,18 +19,12 @@ A client session has two primary uses:
   (running *in the Bokeh server*) before passing them on to a specific
   viewer.
 
-Note About "External" Applications
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-{warning}
-
 '''
 
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -39,16 +32,13 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-
-# External imports
-from six.moves.urllib.parse import quote_plus
+from urllib.parse import quote_plus
 
 # Bokeh imports
 from ..document import Document
-from ..resources import _SessionCoordinates, DEFAULT_SERVER_HTTP_URL
+from ..resources import DEFAULT_SERVER_HTTP_URL, _SessionCoordinates
 from ..util.browser import NEW_PARAM
-from ..util.session_id import generate_session_id
-from ..util.string import format_docstring
+from ..util.token import generate_jwt_token, generate_session_id
 from .util import server_url_for_websocket_url, websocket_url_for_server_url
 
 #-----------------------------------------------------------------------------
@@ -108,7 +98,7 @@ def pull_session(session_id=None, url='default', io_loop=None, arguments=None):
                 can also be `"default"` which will connect to the default app URL
 
         io_loop (``tornado.ioloop.IOLoop``, optional) :
-            The IOLoop to use for the websocket
+            The ``IOLoop`` to use for the websocket
 
         arguments (dict[str, str], optional) :
             A dictionary of key/values to be passed as HTTP request arguments
@@ -120,7 +110,7 @@ def pull_session(session_id=None, url='default', io_loop=None, arguments=None):
 
     Returns:
         ClientSession :
-            A new ClientSession connected to the server
+            A new ``ClientSession`` connected to the server
 
     '''
 
@@ -222,22 +212,22 @@ class ClientSession(object):
     ''' Represents a websocket connection to a server-side session.
 
     Each server session stores a Document, which is kept in sync with the
-    corresponding Document for this ClientSession instance. Udates on either
-    side of the connection will automatically propagate to the other side, as
-    long as the connectiion is open.
+    corresponding Document for this ``ClientSession`` instance. Updates on
+    either side of the connection will automatically propagate to the other
+    side, as long as the connection is open.
 
     ClientSession objects can (and usually should) be used as a context manager
-    so that the sesssion is properly closed:
+    so that the session is properly closed:
 
     .. code-block:: python
 
-        with pull_session(url=app_url) as session:
+        with pull_session(url=app_url) as mysession:
             # customize session here
             script = server_session(session_id=mysession.id, url=app_url)
             return render_template("embed.html", script=script, template="Flask")
 
-    If you do not use ClientSesssion in this way, it is up to you to ensure
-    that ``session.close()`` is called.
+    If you do not use ``ClientSession`` in this way, it is up to you to ensure
+    that ``mysession.close()`` is called.
 
     '''
 
@@ -316,6 +306,11 @@ class ClientSession(object):
         ''' A unique ID for this session. '''
         return self._id
 
+    @property
+    def token(self):
+        ''' A JWT token to authenticate the session. '''
+        return generate_jwt_token(self.id)
+
     # Public methods ----------------------------------------------------------
 
     def connect(self):
@@ -339,20 +334,6 @@ class ClientSession(object):
         '''
         self._connection.force_roundtrip()
 
-    def loop_until_closed(self, suppress_warning=False):
-        ''' Execute a blocking loop that runs and exectutes event callbacks
-        until the connection is closed (e.g. by hitting Ctrl-C).
-
-        While this method can be used to run Bokeh application code "outside"
-        the Bokeh server, this practice is HIGHLY DISCOURAGED for any real
-        use case.
-
-        '''
-        import warnings
-        if not suppress_warning:
-            warnings.warn(_BOKEH_CLIENT_APP_WARNING_FULL)
-        self._connection.loop_until_closed()
-
     def pull(self):
         ''' Pull the server's state and set it as session.document.
 
@@ -468,6 +449,15 @@ class ClientSession(object):
     def _handle_patch(self, message):
         message.apply_to_document(self.document, self)
 
+    def _loop_until_closed(self):
+        ''' Execute a blocking loop that runs and executes event callbacks
+        until the connection is closed (e.g. by hitting Ctrl-C).
+
+        This function is intended to facilitate testing ONLY.
+
+        '''
+        self._connection.loop_until_closed()
+
     def _notify_disconnected(self):
         ''' Called by the ClientConnection we are using to notify us of disconnect.
 
@@ -490,31 +480,6 @@ class ClientSession(object):
 # Private API
 #-----------------------------------------------------------------------------
 
-_BOKEH_CLIENT_APP_WARNING_BODY = """
-The use of `session.loop_until_closed` and `push_session` to run Bokeh
-application code outside a Bokeh server is **HIGHLY DISCOURAGED** for any real
-use.
-
-Running application code outside a Bokeh server with bokeh.client in this way
-has (and always will have) several intrinsic drawbacks:
-
-* Fast binary array transport is NOT available! Base64 fallback is much slower
-* All network traffic is DOUBLED due to extra hop between client and server
-* Server *and* client process must be running at ALL TIMES for callbacks to work
-* App code run outside the Bokeh server is NOT SCALABLE behind a load balancer
-
-The bokeh.client API is recommended to use ONLY for testing, or for customizing
-individual sessions running in a full Bokeh server, before passing on to viewers.
-
-For information about different ways of running apps in a Bokeh server, see:
-
-    http://bokeh.pydata.org/en/latest/docs/user_guide/server.html
-"""
-
-_BOKEH_CLIENT_APP_WARNING_FULL = "\n\n    !!!! PLEASE NOTE !!!!\n" + _BOKEH_CLIENT_APP_WARNING_BODY
-
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
-
-__doc__ = format_docstring(__doc__, warning=_BOKEH_CLIENT_APP_WARNING_BODY)
