@@ -1,7 +1,6 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2018, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
@@ -10,7 +9,7 @@ be reflected to the client side BokehJS library.
 
 As a concrete example, consider a column layout with ``Slider`` and ``Select``
 widgets, and a plot with some tools, an axis and grid, and a glyph renderer
-for circles. A simplified representation oh this document might look like the
+for circles. A simplified representation of this document might look like the
 figure below:
 
 .. figure:: /_images/document.svg
@@ -25,9 +24,7 @@ figure below:
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -35,13 +32,15 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-from collections import defaultdict
-from json import loads
 import sys
+from collections import defaultdict
+from functools import wraps
+from inspect import isclass
+from json import loads
+from typing import Any, Callable, Dict, List
 
 # External imports
 import jinja2
-from six import string_types
 
 # Bokeh imports
 from ..core.enums import HoldPolicy
@@ -50,16 +49,26 @@ from ..core.query import find
 from ..core.templates import FILE
 from ..core.validation import check_integrity
 from ..events import Event
-from ..themes import default as default_theme, built_in_themes
-from ..themes import Theme
+from ..model import Model
+from ..themes import Theme, built_in_themes
+from ..themes import default as default_theme
 from ..util.callback_manager import _check_callback
 from ..util.datatypes import MultiValuedDict
-from ..util.future import wraps
 from ..util.version import __version__
-
-from .events import ModelChangedEvent, RootAddedEvent, RootRemovedEvent, SessionCallbackAdded, SessionCallbackRemoved, TitleChangedEvent
+from .events import (
+    ModelChangedEvent,
+    RootAddedEvent,
+    RootRemovedEvent,
+    SessionCallbackAdded,
+    SessionCallbackRemoved,
+    TitleChangedEvent,
+)
 from .locking import UnlockedDocumentProxy
-from .util import initialize_references_json, instantiate_references_json, references_json
+from .util import (
+    initialize_references_json,
+    instantiate_references_json,
+    references_json,
+)
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -90,6 +99,9 @@ class Document(object):
     of serialization for Bokeh.
 
     '''
+
+    _message_callbacks: Dict[str, List[Callable[[Any], None]]]
+
     def __init__(self, **kwargs):
         self._roots = list()
         self._theme = kwargs.pop('theme', default_theme)
@@ -101,6 +113,7 @@ class Document(object):
         self._all_models_by_name = MultiValuedDict()
         self._all_former_model_ids = set()
         self._callbacks = {}
+        self._message_callbacks = {}
         self._session_destroyed_callbacks = set()
         self._session_callbacks = set()
         self._session_context = None
@@ -111,6 +124,7 @@ class Document(object):
 
         # set of models subscribed to user events
         self._subscribed_models = defaultdict(set)
+        self.on_message("bokeh_event", self.apply_json_event)
 
         self._callback_objs_by_callable = {self.add_next_tick_callback: defaultdict(set),
                                            self.add_periodic_callback: defaultdict(set),
@@ -159,7 +173,7 @@ class Document(object):
 
     @template.setter
     def template(self, template):
-        if not isinstance(template, (jinja2.Template, string_types)):
+        if not isinstance(template, (jinja2.Template, str)):
             raise ValueError("document template must be Jinja2 template or a string")
         self._template = template
 
@@ -192,7 +206,7 @@ class Document(object):
         if self._theme is theme:
             return
 
-        if isinstance(theme, string_types):
+        if isinstance(theme, str):
             try:
                 self._theme = built_in_themes[theme]
             except KeyError:
@@ -331,12 +345,13 @@ class Document(object):
         return self._add_session_callback(cb, callback, one_shot=True, originator=self.add_timeout_callback)
 
     def apply_json_event(self, json):
-        event = loads(json, object_hook=Event.decode_json)
+        event = Event.decode_json(json)
         if not isinstance(event, Event):
-            log.warn('Could not decode event json: %s' % json)
+            log.warning('Could not decode event json: %s' % json)
         else:
-            for obj in self._subscribed_models[event.event_name]:
-                obj._trigger_event(event)
+            subscribed = self._subscribed_models[event.event_name].copy()
+            for model in subscribed:
+                model._trigger_event(event)
 
     def apply_json_patch(self, patch, setter=None):
         ''' Apply a JSON patch object and process any resulting events.
@@ -366,8 +381,8 @@ class Document(object):
 
         # Use our existing model instances whenever we have them
         for obj in references.values():
-            if obj._id in self._all_models:
-                references[obj._id] = self._all_models[obj._id]
+            if obj.id in self._all_models:
+                references[obj.id] = self._all_models[obj.id]
 
         # The model being changed isn't always in references so add it in
         for event_json in events_json:
@@ -376,17 +391,19 @@ class Document(object):
                 if model_id in self._all_models:
                     references[model_id] = self._all_models[model_id]
 
-        initialize_references_json(references_json, references)
+        initialize_references_json(references_json, references, setter)
 
         for event_json in events_json:
+            if event_json['kind'] == 'MessageSent':
+                self._trigger_on_message(event_json["msg_type"], event_json["msg_data"])
 
-            if event_json['kind'] == 'ModelChanged':
+            elif event_json['kind'] == 'ModelChanged':
                 patched_id = event_json['model']['id']
                 if patched_id not in self._all_models:
                     if patched_id not in self._all_former_model_ids:
                         raise RuntimeError("Cannot apply patch to %s which is not in the document" % (str(patched_id)))
                     else:
-                        log.warn("Cannot apply patch to %s which is not in the document anymore" % (str(patched_id)))
+                        log.warning("Cannot apply patch to %s which is not in the document anymore" % (str(patched_id)))
                         break
                 patched_obj = self._all_models[patched_id]
                 attr = event_json['attr']
@@ -506,8 +523,8 @@ class Document(object):
             # leaked. Here we perform a detailed check that the only referrers are expected
             # ones. Otherwise issue an error log message with details.
             referrers = get_referrers(module)
-            referrers = [x for x in referrers if x is not sys.modules]
-            referrers = [x for x in referrers if x is not self._modules]
+            referrers = [x for x in referrers if x is not sys.modules]  # lgtm [py/comparison-using-is]
+            referrers = [x for x in referrers if x is not self._modules]  # lgtm [py/comparison-using-is]
             referrers = [x for x in referrers if not isinstance(x, FrameType)]
             if len(referrers) != 0:
                 log.error("Module %r has extra unexpected referrers! This could indicate a serious memory leak. Extra referrers: %r" % (module, referrers))
@@ -628,7 +645,7 @@ class Document(object):
 
         '''
         if self._hold is not None and self._hold != policy:
-            log.warn("hold already active with '%s', ignoring '%s'" % (self._hold, policy))
+            log.warning("hold already active with '%s', ignoring '%s'" % (self._hold, policy))
             return
         if policy not in HoldPolicy:
             raise ValueError("Unknown hold policy %r" % policy)
@@ -651,6 +668,24 @@ class Document(object):
         for event in events:
             self._trigger_on_change(event)
 
+    def on_message(self, msg_type: str, callback: Callable[[Any], None]) -> None:
+        message_callbacks = self._message_callbacks.get(msg_type, None)
+        if message_callbacks is None:
+            self._message_callbacks[msg_type] = [callback]
+        elif callback not in message_callbacks:
+            message_callbacks.append(callback)
+
+    def remove_on_message(self, msg_type: str, callback: Callable[[Any], None]) -> None:
+        message_callbacks = self._message_callbacks.get(msg_type, None)
+        if message_callbacks is not None and callback in message_callbacks:
+            message_callbacks.remove(callback)
+
+    def _trigger_on_message(self, msg_type: str, msg_data: Any) -> None:
+        message_callbacks = self._message_callbacks.get(msg_type, None)
+        if message_callbacks is not None:
+            for cb in message_callbacks:
+                cb(msg_data)
+
     def on_change(self, *callbacks):
         ''' Provide callbacks to invoke if the document or any Model reachable
         from its roots changes.
@@ -759,7 +794,7 @@ class Document(object):
             None
 
         Raises:
-            ValueError, if the callback was never added or has alraedy been run or removed
+            ValueError, if the callback was never added or has already been run or removed
 
         '''
         self._remove_session_callback(callback_obj, self.add_timeout_callback)
@@ -828,6 +863,8 @@ class Document(object):
             None
 
         '''
+        if isclass(selector) and issubclass(selector, Model):
+            selector = dict(type=selector)
         for obj in self.select(selector):
             for key, val in updates.items():
                 setattr(obj, key, val)
@@ -845,7 +882,7 @@ class Document(object):
         doc_json = self.to_json_string()
         return loads(doc_json)
 
-    def to_json_string(self, indent=None):
+    def to_json_string(self, indent=None) -> str:
         ''' Convert the document to a JSON string.
 
         Args:
@@ -858,7 +895,7 @@ class Document(object):
         '''
         root_ids = []
         for r in self._roots:
-            root_ids.append(r._id)
+            root_ids.append(r.id)
 
         root_references = self._all_models.values()
 
@@ -983,7 +1020,7 @@ class Document(object):
             return False
         if field not in selector:
             return False
-        return isinstance(selector[field], string_types)
+        return isinstance(selector[field], str)
 
     def _notify_change(self, model, attr, old, new, hint=None, setter=None, callback_invoker=None):
         ''' Called by Model when it changes
@@ -1032,11 +1069,11 @@ class Document(object):
         recomputed = {}
         recomputed_by_name = MultiValuedDict()
         for m in new_all_models_set:
-            recomputed[m._id] = m
+            recomputed[m.id] = m
             if m.name is not None:
                 recomputed_by_name.add_value(m.name, m)
         for d in to_detach:
-            self._all_former_model_ids.add(d._id)
+            self._all_former_model_ids.add(d.id)
             d._detach_document()
         for a in to_attach:
             a._attach_document(self)
