@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -24,14 +22,11 @@ log = logging.getLogger(__name__)
 import math
 from collections import namedtuple
 
-# External imports
-from six import string_types
-
 # Bokeh imports
 from .core.enums import Location
-from .models.tools import ProxyToolbar, ToolbarBox
+from .models.layouts import Box, Column, GridBox, LayoutDOM, Row, Spacer, WidgetBox
 from .models.plots import Plot
-from .models.layouts import LayoutDOM, Box, Row, Column, GridBox, Spacer, WidgetBox
+from .models.tools import ProxyToolbar, ToolbarBox
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -76,25 +71,18 @@ def row(*args, **kwargs):
 
     Examples:
 
-        >>> row([plot_1, plot_2])
-        >>> row(children=[widget_box_1, plot_1], sizing_mode='stretch_both')
+        >>> row(plot1, plot2)
+        >>> row(children=[widgets, plot], sizing_mode='stretch_both')
     """
 
     sizing_mode = kwargs.pop('sizing_mode', None)
     children = kwargs.pop('children', None)
 
-    children = _handle_children(*args, children=children)
+    children = _parse_children_arg(*args, children=children)
 
-    row_children = []
-    for item in children:
-        if isinstance(item, LayoutDOM):
-            if sizing_mode is not None and _has_auto_sizing(item):
-                item.sizing_mode = sizing_mode
-            row_children.append(item)
-        else:
-            raise ValueError("""Only LayoutDOM items can be inserted into a row. Tried to insert: %s of type %s""" % (item, type(item)))
+    _handle_child_sizing(children, sizing_mode, widget="row")
 
-    return Row(children=row_children, sizing_mode=sizing_mode, **kwargs)
+    return Row(children=children, sizing_mode=sizing_mode, **kwargs)
 
 
 def column(*args, **kwargs):
@@ -121,25 +109,18 @@ def column(*args, **kwargs):
 
     Examples:
 
-        >>> column([plot_1, plot_2])
-        >>> column(children=[widget_1, plot_1], sizing_mode='stretch_both')
+        >>> column(plot1, plot2)
+        >>> column(children=[widgets, plot], sizing_mode='stretch_both')
     """
 
     sizing_mode = kwargs.pop('sizing_mode', None)
     children = kwargs.pop('children', None)
 
-    children = _handle_children(*args, children=children)
+    children = _parse_children_arg(*args, children=children)
 
-    col_children = []
-    for item in children:
-        if isinstance(item, LayoutDOM):
-            if sizing_mode is not None and _has_auto_sizing(item):
-                item.sizing_mode = sizing_mode
-            col_children.append(item)
-        else:
-            raise ValueError("""Only LayoutDOM items can be inserted into a column. Tried to insert: %s of type %s""" % (item, type(item)))
+    _handle_child_sizing(children, sizing_mode, widget="column")
 
-    return Column(children=col_children, sizing_mode=sizing_mode, **kwargs)
+    return Column(children=children, sizing_mode=sizing_mode, **kwargs)
 
 
 def widgetbox(*args, **kwargs):
@@ -166,17 +147,11 @@ def widgetbox(*args, **kwargs):
     sizing_mode = kwargs.pop('sizing_mode', None)
     children = kwargs.pop('children', None)
 
-    children = _handle_children(*args, children=children)
+    children = _parse_children_arg(*args, children=children)
 
-    col_children = []
-    for item in children:
-        if isinstance(item, LayoutDOM):
-            if sizing_mode is not None and _has_auto_sizing(item):
-                item.sizing_mode = sizing_mode
-            col_children.append(item)
-        else:
-            raise ValueError("""Only LayoutDOM items can be inserted into a widget box. Tried to insert: %s of type %s""" % (item, type(item)))
-    return WidgetBox(children=col_children, sizing_mode=sizing_mode, **kwargs)
+    _handle_child_sizing(children, sizing_mode, widget="widget box")
+
+    return WidgetBox(children=children, sizing_mode=sizing_mode, **kwargs)
 
 
 def layout(*args, **kwargs):
@@ -216,10 +191,10 @@ def layout(*args, **kwargs):
     sizing_mode = kwargs.pop('sizing_mode', None)
     children = kwargs.pop('children', None)
 
-    children = _handle_children(*args, children=children)
+    children = _parse_children_arg(*args, children=children)
 
     # Make the grid
-    return _create_grid(children, sizing_mode)
+    return _create_grid(children, sizing_mode, **kwargs)
 
 def gridplot(children, sizing_mode=None, toolbar_location='above', ncols=None,
              plot_width=None, plot_height=None, toolbar_options=None, merge_tools=True):
@@ -286,7 +261,7 @@ def gridplot(children, sizing_mode=None, toolbar_location='above', ncols=None,
         if not hasattr(Location, toolbar_location):
             raise ValueError("Invalid value of toolbar_location: %s" % toolbar_location)
 
-    children = _handle_children(children=children)
+    children = _parse_children_arg(children=children)
     if ncols:
         if any(isinstance(child, list) for child in children):
             raise ValueError("Cannot provide a nested list when using ncols")
@@ -297,7 +272,7 @@ def gridplot(children, sizing_mode=None, toolbar_location='above', ncols=None,
         children = []
 
     # Make the grid
-    tools = []
+    toolbars = []
     items = []
 
     for y, row in enumerate(children):
@@ -307,7 +282,7 @@ def gridplot(children, sizing_mode=None, toolbar_location='above', ncols=None,
             elif isinstance(item, LayoutDOM):
                 if merge_tools:
                     for plot in item.select(dict(type=Plot)):
-                        tools += plot.toolbar.tools
+                        toolbars.append(plot.toolbar)
                         plot.toolbar_location = None
 
                 if isinstance(item, Plot):
@@ -327,7 +302,8 @@ def gridplot(children, sizing_mode=None, toolbar_location='above', ncols=None,
         return GridBox(children=items, sizing_mode=sizing_mode)
 
     grid = GridBox(children=items)
-    proxy = ProxyToolbar(tools=tools, **toolbar_options)
+    tools = sum([ toolbar.tools for toolbar in toolbars ], [])
+    proxy = ProxyToolbar(toolbars=toolbars, tools=tools, **toolbar_options)
     toolbar = ToolbarBox(toolbar=proxy, toolbar_location=toolbar_location)
 
     if toolbar_location == 'above':
@@ -486,7 +462,7 @@ def grid(children=[], sizing_mode=None, nrows=None, ncols=None):
                 return item
 
         layout = traverse(children, top_level=True)
-    elif isinstance(children, string_types):
+    elif isinstance(children, str):
         raise NotImplementedError
     else:
         raise ValueError("expected a list, string or model")
@@ -572,7 +548,7 @@ class GridSpec(object):
 def _has_auto_sizing(item):
     return item.sizing_mode is None and item.width_policy == "auto" and item.height_policy == "auto"
 
-def _handle_children(*args, **kwargs):
+def _parse_children_arg(*args, **kwargs):
     children = kwargs.get('children')
 
     # Set-up Children from args or kwargs
@@ -589,8 +565,15 @@ def _handle_children(*args, **kwargs):
 
     return children
 
+def _handle_child_sizing(children, sizing_mode, *, widget):
+    for item in children:
+        if not isinstance(item, LayoutDOM):
+            raise ValueError(f"Only LayoutDOM items can be inserted into a {widget}. Tried to insert: {item} of type {type(item)}")
+        if sizing_mode is not None and _has_auto_sizing(item):
+            item.sizing_mode = sizing_mode
 
-def _create_grid(iterable, sizing_mode, layer=0):
+
+def _create_grid(iterable, sizing_mode, layer=0, **kwargs):
     """Recursively create grid from input lists."""
     return_list = []
     for item in iterable:
@@ -606,8 +589,8 @@ def _create_grid(iterable, sizing_mode, layer=0):
                 Tried to insert: %s of type %s""" % (item, type(item))
             )
     if layer % 2 == 0:
-        return column(children=return_list, sizing_mode=sizing_mode)
-    return row(children=return_list, sizing_mode=sizing_mode)
+        return column(children=return_list, sizing_mode=sizing_mode, **kwargs)
+    return row(children=return_list, sizing_mode=sizing_mode, **kwargs)
 
 
 def _chunks(l, ncols):
