@@ -1,40 +1,132 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Renderers for various kinds of annotations that can be added to
 Bokeh plots
 
 '''
-from __future__ import absolute_import
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+import logging # isort:skip
+log = logging.getLogger(__name__)
 
-from six import string_types
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
 
-from ..core.enums import (AngleUnits, Dimension, FontStyle, LegendClickPolicy, LegendLocation,
-                          Orientation, RenderMode, SpatialUnits, VerticalAlign, TextAlign,
-                          TooltipAttachment)
+# Bokeh imports
+from ..core.enums import (
+    AngleUnits,
+    Dimension,
+    FontStyle,
+    LegendClickPolicy,
+    LegendLocation,
+    Orientation,
+    RenderMode,
+    SpatialUnits,
+    TextAlign,
+    TooltipAttachment,
+    VerticalAlign,
+)
 from ..core.has_props import abstract
-from ..core.properties import (Angle, AngleSpec, Auto, Bool, ColorSpec, Datetime, Dict, DistanceSpec, Either,
-                               Enum, Float, FontSizeSpec, Include, Instance, Int, List, NumberSpec, Override,
-                               Seq, String, StringSpec, Tuple, value)
-from ..core.property_mixins import FillProps, LineProps, TextProps
+from ..core.properties import (
+    Angle,
+    AngleSpec,
+    Auto,
+    Bool,
+    ColorSpec,
+    Datetime,
+    Dict,
+    Either,
+    Enum,
+    Float,
+    FontSizeSpec,
+    Include,
+    Instance,
+    Int,
+    List,
+    NumberSpec,
+    Override,
+    PropertyUnitsSpec,
+    Seq,
+    String,
+    StringSpec,
+    Tuple,
+    value,
+)
+from ..core.property_mixins import (
+    FillProps,
+    LineProps,
+    ScalarFillProps,
+    ScalarLineProps,
+    ScalarTextProps,
+    TextProps,
+)
 from ..core.validation import error
-from ..core.validation.errors import BAD_COLUMN_NAME, NON_MATCHING_DATA_SOURCES_ON_LEGEND_ITEM_RENDERERS
+from ..core.validation.errors import (
+    BAD_COLUMN_NAME,
+    NON_MATCHING_DATA_SOURCES_ON_LEGEND_ITEM_RENDERERS,
+)
 from ..model import Model
 from ..util.serialization import convert_datetime_type
-
 from .formatters import BasicTickFormatter, TickFormatter
 from .mappers import ContinuousColorMapper
 from .renderers import GlyphRenderer, Renderer
 from .sources import ColumnDataSource, DataSource
 from .tickers import BasicTicker, ContinuousTicker
 
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'Annotation',
+    'Arrow',
+    'Band',
+    'BoxAnnotation',
+    'ColorBar',
+    'Label',
+    'LabelSet',
+    'Legend',
+    'LegendItem',
+    'PolyAnnotation',
+    'Slope',
+    'Span',
+    'TextAnnotation',
+    'Title',
+    'Tooltip',
+    'ToolbarPanel',
+    'Whisker',
+)
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+# This only exists to prevent a circular import.
+def _DEFAULT_ARROW():
+    from .arrow_heads import OpenHead
+    return OpenHead()
+
+# This only exists to prevent a circular import.
+def _DEFAULT_TEE():
+    from .arrow_heads import TeeHead
+    return TeeHead(level="underlay", size=10)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
 @abstract
 class Annotation(Renderer):
     ''' Base class for all annotation models.
 
     '''
 
-    plot = Instance(".models.plots.Plot", help="""
-    The plot to which this annotation is attached.
-    """)
-
     level = Override(default="annotation")
 
 @abstract
@@ -65,8 +157,8 @@ class LegendItem(Model):
 
     '''
     def __init__(self, *args, **kwargs):
-        super(LegendItem, self).__init__(*args, **kwargs)
-        if isinstance(self.label, string_types):
+        super().__init__(*args, **kwargs)
+        if isinstance(self.label, str):
             # Allow convenience of setting label as a string
             self.label = value(self.label)
 
@@ -125,7 +217,23 @@ class Legend(Annotation):
     when they are drawn.
     """)
 
-    border_props = Include(LineProps, help="""
+    title = String(help="""
+    The title text to render.
+    """)
+
+    title_props = Include(ScalarTextProps, help="""
+    The %s values for the title text.
+    """)
+
+    title_text_font_size = Override(default="10pt")
+
+    title_text_font_style = Override(default="italic")
+
+    title_standoff = Int(5, help="""
+    The distance (in pixels) to separate the title from the legend.
+    """)
+
+    border_props = Include(ScalarLineProps, help="""
     The %s for the legend border outline.
     """)
 
@@ -133,11 +241,11 @@ class Legend(Annotation):
 
     border_line_alpha = Override(default=0.5)
 
-    background_props = Include(FillProps, help="""
+    background_props = Include(ScalarFillProps, help="""
     The %s for the legend background style.
     """)
 
-    inactive_props = Include(FillProps, help="""
+    inactive_props = Include(ScalarFillProps, help="""
     The %s for the legend item style when inactive. These control an overlay
     on the item that can be used to obscure it when the corresponding glyph
     is inactive (e.g. by making it semi-transparent).
@@ -155,13 +263,13 @@ class Legend(Annotation):
 
     inactive_fill_alpha = Override(default=0.7)
 
-    label_props = Include(TextProps, help="""
+    label_props = Include(ScalarTextProps, help="""
     The %s for the legend labels.
     """)
 
     label_text_baseline = Override(default='middle')
 
-    label_text_font_size = Override(default={'value': '10pt'})
+    label_text_font_size = Override(default='10pt')
 
     label_standoff = Int(5, help="""
     The distance (in pixels) to separate the label from its associated glyph.
@@ -261,11 +369,11 @@ class ColorBar(Annotation):
     The title text to render.
     """)
 
-    title_props = Include(TextProps, help="""
+    title_props = Include(ScalarTextProps, help="""
     The %s values for the title text.
     """)
 
-    title_text_font_size = Override(default={'value': "10pt"})
+    title_text_font_size = Override(default="10pt")
 
     title_text_font_style = Override(default="italic")
 
@@ -305,7 +413,7 @@ class ColorBar(Annotation):
     Amount of padding (in pixels) between the color scale and color bar border.
     """)
 
-    major_label_props = Include(TextProps, help="""
+    major_label_props = Include(ScalarTextProps, help="""
     The %s of the major tick labels.
     """)
 
@@ -313,13 +421,13 @@ class ColorBar(Annotation):
 
     major_label_text_baseline = Override(default="middle")
 
-    major_label_text_font_size = Override(default={'value': "8pt"})
+    major_label_text_font_size = Override(default="8pt")
 
     label_standoff = Int(5, help="""
     The distance (in pixels) to separate the tick labels from the color bar.
     """)
 
-    major_tick_props = Include(LineProps, help="""
+    major_tick_props = Include(ScalarLineProps, help="""
     The %s of the major ticks.
     """)
 
@@ -335,7 +443,7 @@ class ColorBar(Annotation):
     main plot area.
     """)
 
-    minor_tick_props = Include(LineProps, help="""
+    minor_tick_props = Include(ScalarLineProps, help="""
     The %s of the minor ticks.
     """)
 
@@ -357,13 +465,13 @@ class ColorBar(Annotation):
 
     bar_line_color = Override(default=None)
 
-    border_props = Include(LineProps, help="""
+    border_props = Include(ScalarLineProps, help="""
     The %s for the color bar border outline.
     """)
 
     border_line_color = Override(default=None)
 
-    background_props = Include(FillProps, help="""
+    background_props = Include(ScalarFillProps, help="""
     The %s for the color bar background style.
     """)
 
@@ -371,13 +479,8 @@ class ColorBar(Annotation):
 
     background_fill_alpha = Override(default=0.95)
 
-# This only exists to prevent a circular import.
-def _DEFAULT_ARROW():
-    from .arrow_heads import OpenHead
-    return OpenHead()
-
 class Arrow(Annotation):
-    ''' Render an arrow as an annotation.
+    ''' Render arrows as an annotation.
 
     '''
 
@@ -496,7 +599,7 @@ class BoxAnnotation(Annotation):
     rendering box annotations on the plot. If unset, use the default y-range.
     """)
 
-    line_props = Include(LineProps, use_prefix=False, help="""
+    line_props = Include(ScalarLineProps, use_prefix=False, help="""
     The %s values for the box.
     """)
 
@@ -504,7 +607,7 @@ class BoxAnnotation(Annotation):
 
     line_color = Override(default="#cccccc")
 
-    fill_props = Include(FillProps, use_prefix=False, help="""
+    fill_props = Include(ScalarFillProps, use_prefix=False, help="""
     The %s values for the box.
     """)
 
@@ -526,16 +629,15 @@ class Band(Annotation):
     ''' Render a filled area band along a dimension.
 
     '''
-
-    lower = DistanceSpec(help="""
+    lower = PropertyUnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
     The coordinates of the lower portion of the filled area band.
     """)
 
-    upper = DistanceSpec(help="""
+    upper = PropertyUnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
     The coordinates of the upper portion of the filled area band.
     """)
 
-    base = DistanceSpec(help="""
+    base = PropertyUnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
     The orthogonal coordinates of the upper and lower values.
     """)
 
@@ -558,7 +660,7 @@ class Band(Annotation):
     rendering annotations on the plot. If unset, use the default y-range.
     """)
 
-    line_props = Include(LineProps, use_prefix=False, help="""
+    line_props = Include(ScalarLineProps, use_prefix=False, help="""
     The %s values for the band.
     """)
 
@@ -566,7 +668,7 @@ class Band(Annotation):
 
     line_color = Override(default="#cccccc")
 
-    fill_props = Include(FillProps, use_prefix=False, help="""
+    fill_props = Include(ScalarFillProps, use_prefix=False, help="""
     The %s values for the band.
     """)
 
@@ -648,20 +750,17 @@ class Label(TextAnnotation):
     distance in screen units from a given data position.
     """)
 
-    # TODO (bev) these should probably not be dataspec properties
-    text_props = Include(TextProps, use_prefix=False, help="""
+    text_props = Include(ScalarTextProps, use_prefix=False, help="""
     The %s values for the text.
     """)
 
-    # TODO (bev) these should probably not be dataspec properties
-    background_props = Include(FillProps, use_prefix=True, help="""
+    background_props = Include(ScalarFillProps, use_prefix=True, help="""
     The %s values for the text bounding box.
     """)
 
     background_fill_color = Override(default=None)
 
-    # TODO (bev) these should probably not be dataspec properties
-    border_props = Include(LineProps, use_prefix=True, help="""
+    border_props = Include(ScalarLineProps, use_prefix=True, help="""
     The %s values for the text bounding box.
     """)
 
@@ -811,7 +910,7 @@ class PolyAnnotation(Annotation):
     rendering box annotations on the plot. If unset, use the default y-range.
     """)
 
-    line_props = Include(LineProps, use_prefix=False, help="""
+    line_props = Include(ScalarLineProps, use_prefix=False, help="""
     The %s values for the polygon.
     """)
 
@@ -819,7 +918,7 @@ class PolyAnnotation(Annotation):
 
     line_color = Override(default="#cccccc")
 
-    fill_props = Include(FillProps, use_prefix=False, help="""
+    fill_props = Include(ScalarFillProps, use_prefix=False, help="""
     The %s values for the polygon.
     """)
 
@@ -850,7 +949,7 @@ class Slope(Annotation):
     rendering annotations on the plot. If unset, use the default y-range.
     """)
 
-    line_props = Include(LineProps, use_prefix=False, help="""
+    line_props = Include(ScalarLineProps, use_prefix=False, help="""
     The %s values for the line.
     """)
 
@@ -896,7 +995,7 @@ class Span(Annotation):
 
     """)
 
-    line_props = Include(LineProps, use_prefix=False, help="""
+    line_props = Include(ScalarLineProps, use_prefix=False, help="""
     The %s values for the span.
     """)
 
@@ -917,6 +1016,12 @@ class Title(TextAnnotation):
     Alignment of the text in its enclosing space, *along* the direction of the text.
     """)
 
+    text_line_height = Float(default=1.0, help="""
+    How much additional space should be allocated for the title. The value is provided
+    as a number, but should be treated as a percentage of font size. The default is
+    100%, which means no additional space will be used.
+    """)
+
     offset = Float(default=0, help="""
     Offset the text by a number of pixels (can be positive or negative). Shifts the text in
     different directions based on the location of the title:
@@ -934,7 +1039,7 @@ class Title(TextAnnotation):
 
     """)
 
-    text_font_size = FontSizeSpec(default=value("10pt"))
+    text_font_size = FontSizeSpec(default="10pt")
 
     text_font_style = Enum(FontStyle, default="bold", help="""
     A style to use for rendering text.
@@ -969,13 +1074,13 @@ class Title(TextAnnotation):
 
     """)
 
-    background_props = Include(FillProps, use_prefix=True, help="""
+    background_props = Include(ScalarFillProps, use_prefix=True, help="""
     The %s values for the text bounding box.
     """)
 
     background_fill_color = Override(default=None)
 
-    border_props = Include(LineProps, use_prefix=True, help="""
+    border_props = Include(ScalarLineProps, use_prefix=True, help="""
     The %s values for the text bounding box.
     """)
 
@@ -1005,17 +1110,12 @@ class Tooltip(Annotation):
     Whether tooltip's arrow should be shown.
     """)
 
-# This only exists to prevent a circular import.
-def _DEFAULT_TEE():
-    from .arrow_heads import TeeHead
-    return TeeHead(level="underlay", size=10)
-
 class Whisker(Annotation):
     ''' Render a whisker along a dimension.
 
     '''
 
-    lower = DistanceSpec(help="""
+    lower = PropertyUnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
     The coordinates of the lower end of the whiskers.
     """)
 
@@ -1023,7 +1123,7 @@ class Whisker(Annotation):
     Instance of ``ArrowHead``.
     """)
 
-    upper = DistanceSpec(help="""
+    upper = PropertyUnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
     The coordinates of the upper end of the whiskers.
     """)
 
@@ -1031,7 +1131,7 @@ class Whisker(Annotation):
     Instance of ``ArrowHead``.
     """)
 
-    base = DistanceSpec(help="""
+    base = PropertyUnitsSpec(default=None, units_type=Enum(SpatialUnits), units_default="data", help="""
     The orthogonal coordinates of the upper and lower values.
     """)
 
@@ -1065,3 +1165,11 @@ class ToolbarPanel(Annotation): # TODO: this shouldn't be an annotation
     toolbar = Instance(".models.tools.Toolbar", help="""
     A toolbar to display.
     """)
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
