@@ -37,6 +37,10 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+import difflib
+import typing as tp
+
 # Bokeh imports
 from ..core.enums import (
     Anchor,
@@ -76,6 +80,7 @@ from ..core.validation.errors import (
     NO_RANGE_TOOL_RANGES,
 )
 from ..model import Model
+from ..util.string import nice_join
 from .annotations import BoxAnnotation, PolyAnnotation
 from .callbacks import Callback
 from .glyphs import MultiLine, Patches, Rect, XYGlyph
@@ -137,6 +142,25 @@ class Tool(Model):
 
     '''
 
+    _known_aliases: tp.ClassVar[tp.Dict[str, tp.Callable[[], "Tool"]]] = {}
+
+    @classmethod
+    def from_string(cls, name: str) -> "Tool":
+        """ Takes a string and returns a corresponding `Tool` instance. """
+        constructor = cls._known_aliases.get(name)
+        if constructor is not None:
+            return constructor()
+        else:
+            known_names = cls._known_aliases.keys()
+            matches, text = difflib.get_close_matches(name.lower(), known_names), "similar"
+            if not matches:
+                matches, text = known_names, "possible"
+            raise ValueError(f"unexpected tool name '{name}', {text} tools are {nice_join(matches)}")
+
+    @classmethod
+    def register_alias(cls, name: str, constructor: tp.Callable[[], "Tool"]) -> None:
+        cls._known_aliases[name] = constructor
+
 @abstract
 class Action(Tool):
     ''' A base class for tools that are buttons in the toolbar.
@@ -1067,7 +1091,7 @@ class HoverTool(Inspection):
 
     .. code-block:: python
 
-        tool.formatters = dict(date="datetime")
+        tool.formatters = {"@date": "datetime"}
 
     will cause format specifications for the "date" column to be interpreted
     according to the "datetime" formatting scheme. The following schemes are
@@ -1490,3 +1514,43 @@ class PolyEditTool(EditTool, Drag, Tap):
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
+
+Tool.register_alias("pan", lambda: PanTool(dimensions="both"))
+Tool.register_alias("xpan", lambda: PanTool(dimensions="width"))
+Tool.register_alias("ypan", lambda: PanTool(dimensions="height"))
+Tool.register_alias("xwheel_pan", lambda: WheelPanTool(dimension="width"))
+Tool.register_alias("ywheel_pan", lambda: WheelPanTool(dimension="height"))
+Tool.register_alias("wheel_zoom", lambda: WheelZoomTool(dimensions="both"))
+Tool.register_alias("xwheel_zoom", lambda: WheelZoomTool(dimensions="width"))
+Tool.register_alias("ywheel_zoom", lambda: WheelZoomTool(dimensions="height"))
+Tool.register_alias("zoom_in", lambda: ZoomInTool(dimensions="both"))
+Tool.register_alias("xzoom_in", lambda: ZoomInTool(dimensions="width"))
+Tool.register_alias("yzoom_in", lambda: ZoomInTool(dimensions="height"))
+Tool.register_alias("zoom_out", lambda: ZoomOutTool(dimensions="both"))
+Tool.register_alias("xzoom_out", lambda: ZoomOutTool(dimensions="width"))
+Tool.register_alias("yzoom_out", lambda: ZoomOutTool(dimensions="height"))
+Tool.register_alias("click", lambda: TapTool(behavior="inspect"))
+Tool.register_alias("tap", lambda: TapTool())
+Tool.register_alias("crosshair", lambda: CrosshairTool())
+Tool.register_alias("box_select", lambda: BoxSelectTool())
+Tool.register_alias("xbox_select", lambda: BoxSelectTool(dimensions="width"))
+Tool.register_alias("ybox_select", lambda: BoxSelectTool(dimensions="height"))
+Tool.register_alias("poly_select", lambda: PolySelectTool())
+Tool.register_alias("lasso_select", lambda: LassoSelectTool())
+Tool.register_alias("box_zoom", lambda: BoxZoomTool(dimensions="both"))
+Tool.register_alias("xbox_zoom", lambda: BoxZoomTool(dimensions="width"))
+Tool.register_alias("ybox_zoom", lambda: BoxZoomTool(dimensions="height"))
+Tool.register_alias("save", lambda: SaveTool())
+Tool.register_alias("undo", lambda: UndoTool())
+Tool.register_alias("redo", lambda: RedoTool())
+Tool.register_alias("reset", lambda: ResetTool())
+Tool.register_alias("help", lambda: HelpTool())
+Tool.register_alias("box_edit", lambda: BoxEditTool())
+Tool.register_alias("point_draw", lambda: PointDrawTool())
+Tool.register_alias("poly_draw", lambda: PolyDrawTool())
+Tool.register_alias("poly_edit", lambda: PolyEditTool())
+Tool.register_alias("hover", lambda: HoverTool(tooltips=[
+    ("index", "$index"),
+    ("data (x, y)", "($x, $y)"),
+    ("screen (x, y)", "($sx, $sy)"),
+]))
