@@ -689,4 +689,7 @@ BokehTornado.__doc__ = format_docstring(
 # See https://github.com/bokeh/bokeh/issues/9507
 if sys.platform == 'win32' and sys.version_info[:3] >= (3, 8, 0):
     import asyncio
-    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
+    if type(asyncio.get_event_loop_policy()) is asyncio.WindowsProactorEventLoopPolicy:
+        # WindowsProactorEventLoopPolicy is not compatible with tornado 6
+        # fallback to the pre-3.8 default of WindowsSelectorEventLoopPolicy
+        asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
